/**************************************************************************
 *                                                                        *
 * Copyright (c) 1996 Michael Richmond and Richard Treffers               *
 *                                                                        *
 *                    This software may be copied and distributed for     *
 *                    educational, research and not for profit services   *
 *                    provided that this copyright and statement are      *
 *                    included in all such copies.                        *
 *                                                                        *
 **************************************************************************/


	/*
	 * This file contains a "driver" for initiating a simple
	 * X Windows plotting program; it takes input from the
	 * user and calls functions in "xplotlib.c" to do the work.
	 *
	 * MWR 8/18/96
	 *
	 * Check the value of "fgets()" in an endless loop -- not to fix
	 * program behavior, just to get rid of compiler warning.  
	 *
	 * MWR 3/10/2017
	 */
	 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <signal.h>
#include "xplotlib.h"

typedef void (*PFV)(int);

static PFV
trap(char *s);


char progname[50];

int
main(argc, argv)
int argc;
char *argv[];
{
	char s[80], command[80];
	int child_id;

	strcpy(progname, argv[0]);
	if ((argc > 1) && (strcmp(argv[1], "help") == 0)) {
		printf("usage: %s [command_file] [help] \n", progname);
		printf("enter 'help' after starting %s\n", progname);
		exit(0);
	}

	signal(SIGINT, (PFV) trap);
	xplot_init();

	if ((child_id = fork()) == 0) {			/* spin off the graphics */
		if (argc>1) {				/* added rrt */
			char buf[80];
			sprintf(buf,"run %s",argv[1]);	
			sleep(1);						/* !!! kludge */
			xplot_transmit(buf);
		}
		while(1){		/* while the graphics are running */
			if (fgets(s, 80, stdin) != NULL) {
				xplot_transmit(s);
			}
		}
	}

	xplot_clear();

	printf("Welcome to %s - type 'help' for help\n", progname);
	while (1) {
		printf(">");
		fflush(stdout);
		command[0] = '\0';
		xplot_event(s, XPLOT_EVENT_LOOP);
		sscanf(s, "%s", command);
		if (command[0] == '\0') {
			continue;
		}
		if (xplot_command(s) == XPLOT_QUIT) {
			break;
		}
	}

	exit(0);
}


static PFV
trap(char *s)
{
	fprintf(stderr, "%s\n", s);
	xplot_quit();
	exit(1);
}


